@@ -1,36 +1,18 @@
-use std::sync::{
-    atomic::{AtomicBool, Ordering},
-    Arc, Mutex,
-};
-
-use std::time::Duration;
-
-use super::utils::{
-    check_args, fetch_symbols_retry, get_candlestick_intervals, get_connection_interval_ms,
-    get_send_interval_ms,
-};
-use crate::utils::WS_LOCKS;
+use super::{crawl_event, utils::fetch_symbols_retry};
 use crate::{msg::Message, MessageType};
 use crypto_markets::MarketType;
 use crypto_rest_client::*;
 use crypto_ws_client::*;
-use log::*;
+use std::sync::{Arc, Mutex};
 
 const EXCHANGE_NAME: &str = "okex";
-// https://www.okex.com/docs/zh/#question-public
-// How many subscriptions per websocket connection?
-// The total size of subscription command should not exceed 4096 bytes.
-const MAX_SUBSCRIPTIONS_PER_CONNECTION: usize = 256;
-
-#[rustfmt::skip]
-gen_crawl_event!(crawl_trade_internal, OkexWSClient, MessageType::Trade, subscribe_trade);
 
 pub(crate) fn crawl_trade(
     market_type: MarketType,
     symbols: Option<&[String]>,
     on_msg: Arc<Mutex<dyn FnMut(Message) + 'static + Send>>,
     duration: Option<u64>,
-) -> Option<std::thread::JoinHandle<()>> {
+) {
     if market_type == MarketType::EuropeanOption
         && (symbols.is_none() || symbols.unwrap().is_empty())
     {
@@ -54,27 +36,18 @@ pub(crate) fn crawl_trade(
         let ws_client = OkexWSClient::new(on_msg_ext, None);
         ws_client.subscribe(&channels);
         ws_client.run(duration);
-        None
     } else {
-        crawl_trade_internal(market_type, symbols, on_msg, duration)
+        crawl_event(
+            EXCHANGE_NAME,
+            MessageType::Trade,
+            market_type,
+            symbols,
+            on_msg,
+            duration,
+        );
     }
 }
 
-#[rustfmt::skip]
-gen_crawl_event!(crawl_l2_event, OkexWSClient, MessageType::L2Event, subscribe_orderbook);
-
-#[rustfmt::skip]
-gen_crawl_event!(crawl_bbo, OkexWSClient, MessageType::BBO, subscribe_bbo);
-
-#[rustfmt::skip]
-gen_crawl_event!(crawl_l2_topk, OkexWSClient, MessageType::L2TopK, subscribe_orderbook_topk);
-
-#[rustfmt::skip]
-gen_crawl_event!(crawl_ticker, OkexWSClient, MessageType::Ticker, subscribe_ticker);
-
-#[rustfmt::skip]
-gen_crawl_candlestick!(crawl_candlestick, OkexWSClient);
-
 #[allow(clippy::unnecessary_unwrap)]
 pub(crate) fn crawl_funding_rate(
     market_type: MarketType,
