@@ -77,3 +77,8 @@ fn test_crawl_ticker(market_type: MarketType, symbol: &str) {
         MessageType::Ticker
     )
 }
+
+#[test_case(MarketType::Spot)]
+fn test_crawl_candlestick(market_type: MarketType) {
+    gen_test_crawl_candlestick!(EXCHANGE_NAME, market_type)
+}
